; Pearl VII by Philipp Lenssen, www.outer-court.com

Include "header.bb"
Include "key_constants.bb"
Include "3d_constants.bb"
Include "sprite_routines.bb"
Include "sprite_class_behavior.bb"
Include "sprite_data.bb"
Include "editor.bb"
Include "cache.bb"
Include "misc.bb"
Include "segmented_cube.bb"

Global game.typeGame = New typeGame
main

Function main()
	Graphics3D gScreenWidth, gScreenHeight, gColorDepth
	ClearTextureFilters	
	game\editMode = False
	initGame game
	
	If game\editMode Then
		createEditor
	Else
		editorLoadLevel game\level
		createPlayer
	End If

	createTerrainSprite
	createLights
	createCameraSprite
	createBackground
		
	timer = CreateTimer(gTimerFrequency)
	While Not game\over
		handleInput game
		handleGameTime game
		handleSprites game
		
		UpdateWorld
		handleCollisionReaction
		
		SetBuffer BackBuffer()
		RenderWorld
		showMessage
		Flip
		WaitTimer timer
	Wend

	cleanUp
	End
End Function

Function initGame(game.typeGame)
	game\level = 1
	game\over = False
	game\time = 0
	game\timeMax = 1

	game\soundOn = True
	game\musicOn = True
	game\musicFile$ = "sound\caravan.mid"
	game\musicChannel = PlayMusic(game\musicFile$)

	messageFont = LoadFont("Arial Narrow", 20, True, False, False)
	SetFont messageFont
	Color 0, 0, 0
End Function

Function handleGameTime(game.typeGame)
	game\time = game\time + 1
	If game\time > game\timeMax Then
		game\time = 0
		game\timeEvent = True
	Else
		game\timeEvent = False
	End If	
End Function

Function handleInput(game.typeGame)
	If KeyDown(gKeyEscape) Then
		game\over = True
	End If
End Function

Function soundEffect(fileName$)
	If game\soundOn Then
		PlaySound getSoundFromCache("sound/" + fileName$ + ".wav", gDefaultVolume#)
	End If
End Function

Function sendMessage(message$)
	game\message$ = message$
End Function

Function showMessage()
	If game\message$ <> 0 Then
		showShadowedText game\message$
		game\message$ = ""
	End If
End Function

Function showShadowedText(thisText$)
	xBase = 10
	yBase = gScreenBottom - 20
	Color 0, 0, 0
	For xOff = -1 To 1 Step 2
		For yOff = -1 To 1 Step 2
			Locate xBase + xOff, yBase + yOff
			Print thisText$
		Next
	Next
	Locate xBase, yBase
	Color 255, 255, 255
	Print thisText$
End Function

Function cleanUp()
	For sprite.typeSprite = Each typeSprite
		Select sprite\class
			Case classCamera
			Case classLight
			Case classTerain
			Case classLight
			Case classAmbientLight
			Case classShadow
			Default
				FreeEntity sprite\body
				Delete sprite
		End Select
	Next
End Function