Function initSprite(sprite.typeSprite)
	sprite\colSpeed = 10
	sprite\alpha# = nothing
	sprite\alphaTarget# = nothing
	sprite\alphaSpeed# = .05
	sprite\scaleSpeed# = .07
	sprite\angleSpeed# = 10
	sprite\positionSpeed# = .5
	sprite\energy = 100
	sprite\isPhysical = True
	sprite\isMovable = False

	clearSpeed sprite
	clearColor sprite
	clearColorTarget sprite
	clearScaleTarget sprite
	clearPositionTarget sprite
	clearAngleTarget sprite
	clearStickTo sprite
End Function

Function createPlayer()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateSphere()
	sprite\class = classPlayer
	setPositionXYZ sprite, -2, -2, 8
	setScaleAll sprite, .4
	EntityShininess sprite\body, .6

	sprite\alpha# = .8
	
	setTextureWithFilter sprite, "texture\glass.bmp", gTextureSphericalReflectionMap
	TextureBlend sprite\texture, gBlendMultiply

	sprite\canFall = True
	sprite\isMovable = True
	setSpeedMaxAll sprite, .4
	setSpeedMax sprite, dY, 1.5
	setSpeedBreakAll sprite, .04;.05
	setSpeedStepAll sprite, .1

	setSpriteColorAll sprite, paletteMax - 30
	setSpriteColorTargetAll sprite, paletteMax - 30
	sprite\colSpeed = 20
	sprite\energy = 50

	EntityRadius sprite\body, sprite\scale#[dMin]
	EntityType sprite\body, sprite\class
	createPersonalLight sprite\body
End Function

Function createTile(editor.typeSprite)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classTile
	sprite\subClass = editor\subClass

	setScaleAfter = True
	setPositionAfter = True
	Select sprite\subClass
		Case 0
			sprite\body = CreateCylinder()
			filter = gTextureColor + gTextMipmapped
		Case 4
			sprite\body = createSegCube(4, 0, .2); CreateCube()
			filter = gTextureColor + gTextMipmapped
			copyPosition editor, sprite
			sprite\position#[dY] = sprite\position#[dY] - .6
			
			sprite\positionTargetFollow = 1
			setPositionTargetBySprite sprite, 0, editor, 0, -.9, 0
			setPositionTargetBySprite sprite, 1, editor, 0, 5, 0
			setSpeedMaxAll sprite, .3
			setSpeedStepAll sprite, .07
			setSpeedBreakAll sprite, .01
			sprite\isMovable = True
			
			setScaleAll sprite, 1
			
			setPositionAfter = False
			setScaleAfter = False
		Case 5
			sprite\body = CreateCube()
			filter = gTextureMasked
			FlipMesh sprite\body
		Case 6
			sprite\body = CreateSphere()
			filter = gTextureSphericalReflectionMap
		Default
			sprite\body = CreateCube()
			filter = gTextureColor + gTextMipmapped
	End Select
	fileName$ = "texture\" + sprite\subClass + ".bmp"
	setTextureWithFilter sprite, fileName$, filter

	EntityShininess sprite\body, .5

	If setPositionAfter Then
		copyPosition editor, sprite
	End If

	If setScaleAfter Then
		copyScale editor, sprite
	End If

	copyAngle editor, sprite

	u# = ( 1 / (sprite\scale#[dX] * 2) )
	v# = ( 1 / (sprite\scale#[dX] * 2) )
	ScaleTexture sprite\texture, u#, v#
	u# = sprite\scale#[dX] - .5
	v# = sprite\scale#[dX] - .5
	PositionTexture sprite\texture, u#, v#

	EntityType sprite\body, sprite\class
End Function

Function createParticle(x#, y#, z#, doMove, doRandomize)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateSprite()
	sprite\class = classParticle

	sprite\isMovable = True

	setPositionXYZ sprite, x#, y#, z#
	
	If doRandomize Then
		randomizePosition sprite, 1
	End If

	If doMove Then
		setSpeedMaxAll sprite, 3
		setSpeedBreakAll sprite, .1
		setSpeedStepAll sprite, .1
		sprite\speed#[dY] = 1 + Rnd(1)
		sprite\canFall = True
	End If
	
	setScaleAll sprite, .04

	sprite\alpha# = 1
	sprite\alphaSpeed# = .1
	sprite\alphaTarget = .1 
	
	sprite\energy = 7
	sprite\energyDirection = -1
	setSpriteColorRGB sprite, 255, 255, 255
	setSpriteColorTargetAll sprite, 0
	sprite\colSpeed = 26
End Function

Function createShadowParticle(x#, y#, z#)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateSprite()
	sprite\class = classParticle

	sprite\isMovable = True

	setPositionXYZ sprite, x#, y#, z#
	setScaleAll sprite, .04

	sprite\alpha# = 1
	sprite\alphaSpeed# = .3
	sprite\alphaTarget = .01
	
	sprite\energy = 6
	sprite\energyDirection = -1
	setSpriteColorAll sprite, 0
End Function

Function createTerrainSprite()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateTerrain(gTerrainSize)
	sprite\class = classTerrain
	setPosition sprite, dY, -9.6
	;setPosition sprite, dX, -30

	setPosition sprite, dX, -30
	setPosition sprite, dZ, -10

	setSpriteColorRGB sprite, 0, 100, 200
	TerrainDetail sprite\body, 150 ; 250
	sprite\timeMax = 5
	setTexture sprite, "texture\water_2.bmp"
	
	EntityAlpha sprite\body, .7

	TerrainShading sprite\body, True

	sprite\isPhysical = False
End Function

Function createCameraSprite()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classCamera
	sprite\body = CreateCamera()
	;CameraViewport sprite\body, gScreenLeft, gScreenTop, gScreenWidth, gScreenHeight
	CameraZoom sprite\body, 2.5

	CameraFogColor sprite\body, 0, 40, 90
	CameraFogMode sprite\body, True
	CameraFogRange sprite\body, 24, 55
	CameraRange sprite\body, .5, 60

	setAngleXYZ sprite, 35, 55, 0
	setPositionXYZ sprite, 17, 9, 0

	sprite\isPhysical = False
End Function

Function createBackground()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateSphere()
	sprite\class = classBackground
	setPositionXYZ sprite, -4, 4, 14
	setScaleAll sprite, 30
	EntityShininess sprite\body, .6
	setTexture sprite, "texture\water.bmp"
	TextureBlend sprite\texture, gBlendMultiply
	FlipMesh sprite\body
	sprite\angleSpeed# = .4
End Function

Function createPlayerShade(reference.typeSprite)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classEffect
	sprite\body = CreateSphere()

	sprite\energy = 8
	sprite\energyDirection = -1
	
	copyPosition reference, sprite
	copyColor reference, sprite
	copyScale reference, sprite
	sprite\texture = reference\texture
	setScaleTargetAll sprite, .1
	sprite\scaleSpeed# = .005

	sprite\alpha# = .6
	sprite\alphaTarget# = .01
	sprite\alphaSpeed# = .08

	setSpriteColorAll sprite, paletteMax ;/ 2
	setSpriteColorTargetAll sprite, paletteMin
	sprite\colSpeed = 30
End Function

Function createExplosion(reference.typeSprite)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classEffect
	sprite\body = CreateSphere()
	setTexture sprite, "texture\chaos.bmp"

	sprite\energy = 6
	sprite\energyDirection = -1
	
	copyPosition reference, sprite

	setScaleAll sprite, reference\scale#[dMin] + 1.2
	setScaleTargetAll sprite, .1
	sprite\scaleSpeed# = .15

	sprite\alpha# = .6
	sprite\alphaTarget# = .01
	sprite\alphaSpeed# = .08

	setSpriteColorAll sprite, paletteMax ;/ 2
	setSpriteColorTargetAll sprite, paletteMin
	sprite\colSpeed = 30
End Function

Function createLights()
	createLight1
	createLight2
	createLight3
	createAmbientLight
End Function

Function createAmbientLight()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classAmbientLight
	sprite\isPhysical = False

	setSpriteColorRGB sprite, 255, 160, 120

	;setSpriteColorRGB sprite, 0, 0, 0
	;setSpriteColorTargetRGB sprite, 255, 160, 120
	;sprite\colSpeed = 20
End Function

Function createLight1()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateLight(gLightPoint)
	sprite\class = classLight
	sprite\isPhysical = False

	setSpriteColorRGB sprite, 255, 255, 255
	setPositionXYZ sprite, 0, 65, 5
	setPositionXYZ sprite, 2, -7, -8
	setAngleXYZ sprite, 90, 0, 0

	LightRange sprite\body, 40
End Function

Function createLight2()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateLight(gLightArea)
	sprite\class = classLight
	sprite\isPhysical = False

	setSpriteColorRGB sprite, 255, 200, 55
	setPositionXYZ sprite, 0, 65, 5
End Function
	
Function createLight3()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateLight(gLightArea)
	sprite\class = classLight
	sprite\isPhysical = False

	setSpriteColorRGB sprite, 55, 100, 255
	setPositionXYZ sprite, 0, 65, 5
	setAngleXYZ sprite, 0, 90, 0
End Function

Function createPersonalLight(personalBody)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateLight(gLightPoint)
	sprite\isPhysical = False
	sprite\class = classLight

	setSpriteColorRGB sprite, -90, -50, -20
	LightRange sprite\body, 3
	sprite\stickToOffset#[dY] = 1.1
	setAngleXYZ sprite, 90, 0, 0 ; 35, 55, 0

	sprite\stickToBody = personalBody
End Function

Function createEditor()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\class = classTile
	sprite\body = CreateCube()
	sprite\isPhysical = True
	
	sprite\class = classEditor
	
	setPositionXYZ sprite, -5, -9, 6
	setScaleAll sprite, .5 ; .6
	sprite\subClass = 0
	setTexture sprite, "texture\" + sprite\subClass + ".bmp"
End Function

Function spriteCreateMirror()
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateMirror()
	sprite\isPhysical = False
	setPosition sprite, dY, -9.5
End Function