Function handleSprites(game.typeGame)
	playerFound = False
	For sprite.typeSprite = Each typeSprite
		handleSingleSprite game, sprite
		If sprite\class = classPlayer Then
			playerFound = True
		End If
	Next
	
	If Not playerFound Then
		If Not game\editMode Then
			game\over = True
		End If
	End If

	handleCollisions
	
	For sprite.typeSprite = Each typeSprite
		If sprite\energy < 0 Then
			FreeEntity sprite\body
			Delete sprite
		End If
	Next
End Function

Function handleCollisions()
	Collisions classPlayer, classTile, gCollisionMethodSphereToPolygon, gCollisionResponseStop
	For sprite.typeSprite = Each typeSprite
		For d = dMin To dMax
			sprite\didCollide[d] = False
		Next
	Next
End Function

Function handleSingleSprite(game.typeGame, sprite.typeSprite)
	handleTimer sprite
	handleEnergy sprite
	If sprite\isPhysical Then
		followScaleTarget sprite
		adaptScale sprite
		handleAlpha sprite
	End If

	isLight = (sprite\class = classLight Or sprite\class = classAmbientLight)
	If sprite\isPhysical Or isLight Then
		handleColor sprite
	End If
	
	If 	sprite\canFall Then
		adaptFall sprite
	End If
	handleClass game, sprite

	If Not sprite\class = classAmbientLight Then
		handleAngle sprite
		handleSpeed sprite
		handleStickToTarget sprite
		followPositionTarget sprite
		adaptPosition sprite
	End If
End Function

Function handleCollisionReaction()
	For sprite.typeSprite = Each typeSprite
		If sprite\class = classPlayer Then

			If EntityCollided(sprite\body, classTile) Then

				For k = 1 To CountCollisions(sprite\body)
				
					cx# = CollisionX(sprite\body, k)
					cy# = CollisionY(sprite\body, k)
					cz# = CollisionZ(sprite\body, k)
					nx# = CollisionNX(sprite\body, k)
					ny# = CollisionNY(sprite\body, k)
					nz# = CollisionNZ(sprite\body, k)
					
					sprite\didCollide[dX] = (nx# <> 0)
					sprite\didCollide[dY] = (ny# <> 0)
					sprite\didCollide[dZ] = (nz# <> 0)

					collided = sprite\didCollide[dX] Or sprite\didCollide[dY] Or sprite\didCollide[dZ]

					If collided Then
						otherBody = CollisionEntity(sprite\body, k)
						otherSprite.typeSprite = getSpriteOfBody.typeSprite(otherBody)

						If sprite\didCollide[dY] Then
							If otherSprite\isMovable Then
								sprite\speed#[dY] = sprite\speed#[dY] * -1
								sprite\position#[dY] = EntityY#(sprite\body)
								sprite\position#[dY] = sprite\position#[dY] + 1
								PositionEntity sprite\body, sprite\position#[dX], sprite\position#[dY], sprite\position#[dZ]
								sprite\didCollide[dY] = True
							Else
								If Abs(sprite\speed#[dY]) < .3 Then
									sprite\speed#[dY] = sprite\speed#[dY] * 2
								End If
								sprite\speed#[dY] = sprite\speed#[dY] * -1
								sprite\didCollide[dY] = True
							End If
						Else
							If sprite\didCollide[dX] Then
								sendMessage "Collision X"
								sprite\speed#[dX] = sprite\speed#[dX] * -1
							End If
							If sprite\didCollide[dZ] Then
								sendMessage "Collision Z: " + cz#
								sprite\speed#[dZ] = sprite\speed#[dZ] * -1
							End If
						End If
	
					End If

				Next

			Else
				For d = dMin To dMax
					sprite\didCollide[d] = False
				Next
			End If
			
		End If
	Next
	ClearCollisions

	For sprite.typeSprite = Each typeSprite
		If sprite\class = classPlayer Then
			sprite\position#[dX] = EntityX#(sprite\body)
			sprite\position#[dY] = EntityY#(sprite\body)
			sprite\position#[dZ] = EntityZ#(sprite\body)
		End If
	Next
End Function

Function handleAngle(sprite.typeSprite)
	followAngleTarget sprite
	keepLimitsAngle sprite
	adaptAngle sprite
End Function

Function verboseAngle$(sprite.typeSprite)
	verbose$ = "AngleXYZ: "
	For d = dMin To dMax
		verbose$ = verbose$ + sprite\angle#[d] + " | "
	Next
	
	Return verbose$
End Function

Function keepLimitsAngle(sprite.typeSprite)
	For d = dMin To dMax
		If sprite\angle#[d] <> nothing Then
			If sprite\angle#[d] > gAngleMax Then
				sprite\angle#[d] = gAngleMin
			ElseIf sprite\angle#[d] < gAngleMin Then
				sprite\angle#[d] = gAngleMax
			End If
		End If

		If sprite\angleTarget#[d] <> nothing Then
			If sprite\angleTarget#[d] > gAngleMax Then
				sprite\angleTarget#[d] = gAngleMin
			ElseIf sprite\angleTarget#[d] < gAngleMin Then
				sprite\angleTarget#[d] = gAngleMax
			End If
		End If
	Next
End Function

Function handleStickToTarget(sprite.typeSprite)
	If sprite\stickToBody <> nothing Then
		target.typeSprite = getSpriteOfBody(sprite\stickToBody)
		For d = dMin To dMax
			If d = dY Then
				distance# = Abs( (target\position#[d] + sprite\stickToOffset#[d]) - sprite\position#[d] )
				distanceOk = (distance# > .25)
			Else
				distanceOk = True
			End If
			If distanceOk Then
				sprite\position#[d] = target\position#[d] + sprite\stickToOffset#[d]
			End If
		Next
	End If
End Function

Function clearStickTo(sprite.typeSprite)
	sprite\stickToBody = nothing
	For d = dMin To dMax
		sprite\stickToOffset#[d] = 0
	Next
End Function

Function followScaleTarget(sprite.typeSprite)
	For d = dMin To dMax
		If sprite\scaleTarget#[d] <> nothing Then
			If sprite\scale#[d] < sprite\scaleTarget[d] Then
				sprite\scale#[d] = sprite\scale#[d] + sprite\scaleSpeed#
				If sprite\scale#[d] > sprite\scaleTarget#[d] Then
					sprite\scale#[d] = sprite\scaleTarget#[d]
				End If
			ElseIf sprite\scale#[d] > sprite\scaleTarget#[d] Then
				sprite\scale#[d] = sprite\scale#[d] - sprite\scaleSpeed#
				If sprite\scale#[d] < sprite\scaleTarget#[d] Then
					sprite\scale#[d] = sprite\scaleTarget#[d]
				End If
			End If
		End If
	Next
End Function

Function followAngleTarget(sprite.typeSprite)
	For d = dMin To dMax
		If sprite\angleTarget#[d] <> nothing Then
			If sprite\angle#[d] < sprite\angleTarget[d] Then
				sprite\angle#[d] = sprite\angle#[d] + sprite\angleSpeed#
				If sprite\angle#[d] > sprite\angleTarget#[d] Then
					sprite\angle#[d] = sprite\angleTarget#[d]
				End If
			ElseIf sprite\angle#[d] > sprite\angleTarget#[d] Then
				sprite\angle#[d] = sprite\angle#[d] - sprite\angleSpeed#
				If sprite\angle#[d] < sprite\angleTarget#[d] Then
					sprite\angle#[d] = sprite\angleTarget#[d]
				End If
			End If
		End If
	Next
End Function

Function handleAlpha(sprite.typeSprite)
	If sprite\alpha# <> nothing Then
		followAlphaTarget sprite
		EntityAlpha sprite\body, sprite\alpha#
	End If
End Function

Function followAlphaTarget(sprite.typeSprite)
	If sprite\alphaTarget# <> nothing Then
		If sprite\alpha# < sprite\alphaTarget# Then
			sprite\alpha# = sprite\alpha# + sprite\alphaSpeed#
			If sprite\alpha# > alphaMax# Then
				sprite\alpha# = alphaMax#
			End If
		ElseIf sprite\alpha# > sprite\alphaTarget# Then
			sprite\alpha# = sprite\alpha# - sprite\alphaSpeed#
			If sprite\alpha# < alphaMin# Then
				sprite\alpha# = alphaMin#
			End If
		End If
	End If
End Function

Function handleColor(sprite.typeSprite)
	followColorTarget sprite
	If hasColor(sprite) Then
		If sprite\class = classLight Then
			LightColor sprite\body, sprite\col[cRed], sprite\col[cGreen], sprite\col[cBlue]
		ElseIf sprite\class = classAmbientLight Then
			AmbientLight sprite\col[cRed], sprite\col[cGreen], sprite\col[cBlue]
		Else
			EntityColor sprite\body, sprite\col[cRed], sprite\col[cGreen], sprite\col[cBlue]
		End If
	End If
End Function

Function hasColor(sprite.typeSprite)
	hasColor = False
	For c = colMin To colMax
		If sprite\col[cRed] <> nothing Then
			hasColor = True
			;Exit For
		End If
	Next

	Return hasColor
End Function

Function followColorTarget(sprite.typeSprite)
	For c = colMin To colMax
		If sprite\colTarget[c] <> nothing Then
			If sprite\col[c] < sprite\colTarget[c] Then
				sprite\col[c] = sprite\col[c] + sprite\colSpeed
				If sprite\col[c] > paletteMax Then
					sprite\col[c] = paletteMax
				ElseIf sprite\col[c] > sprite\colTarget[c] Then
					sprite\col[c] = sprite\colTarget[c]
				End If
			ElseIf sprite\col[c] > sprite\colTarget[c] Then
				sprite\col[c] = sprite\col[c] - sprite\colSpeed
				If sprite\col[c] < paletteMin Then
					sprite\col[c] = paletteMin
				ElseIf sprite\col[c] < sprite\colTarget[c] Then
					sprite\col[c] = sprite\colTarget[c]
				End If
			End If
		End If
	Next
End Function

Function handleEnergy(sprite.typeSprite)
	If sprite\energyDirection <> 0 Then
		sprite\energy = sprite\energy + sprite\energyDirection
	End If
End Function

Function handleTimer(sprite.typeSprite)
	sprite\time = sprite\time + 1
	If sprite\time > sprite\timeMax Then
		sprite\time = 0
		sprite\timeEvent = True
	Else
		sprite\timeEvent = False
	End If	
End Function

Function verboseColor$(sprite.typeSprite)
	verbose$ = ""

	verbose$ = verbose$ + "RGB("
	verbose$ = verbose$ + verboseNumber$(sprite\col[cRed]) + ", "
	verbose$ = verbose$ + verboseNumber$(sprite\col[cGreen]) + ", "
	verbose$ = verbose$ + verboseNumber$(sprite\col[cBlue]) + ")"
	
	Return verbose$
End Function

Function verboseNumber$(n)
	verbose$ = ""
	If n = nothing Then
		verbose$ = " "
	Else
		verbose$ = n
	End If
	
	Return verbose$
End Function

Function getSpriteTopSpeed#(sprite.typeSprite)
	topSpeed# = 0
	For d = dMin To dMax
		thisSpeed# = Abs(sprite\speed#[d])
		If thisSpeed# > topSpeed# Then
			topSpeed# = thisSpeed#
		End If
	Next
	
	Return topSpeed#
End Function

Function randomizePosition(sprite.typeSprite, amount#)
	For d = dMin To dMax
		sprite\position#[d] = sprite\position#[d] + Rnd(amount# / 2) - amount#
	Next
End Function

Function spriteCopyPosition(original.typeSprite, sprite.typeSprite)
	For d = dMin To dMax
		sprite\position#[d] = original\position#[d]
	Next
End Function

Function spriteCopySpeed(original.typeSprite, sprite.typeSprite)
	For d = dMin To dMax
		sprite\speed#[d] = original\speed#[d]
	Next
End Function

Function getSprite.typeSprite(ofClass)
	For sprite.typeSprite = Each typeSprite
		If sprite\class = ofClass Then
			Return sprite
		End If
	Next
End Function

Function getSpriteOfBody.typeSprite(ofBody)
	For sprite.typeSprite = Each typeSprite
		If sprite\body = ofBody Then
			Return sprite
		End If
	Next
End Function

Function handleSpeed(sprite.typeSprite)
	If sprite\isMovable Then
		For d = dMin To dMax
			spriteBreakSpeed sprite, d
			spriteAdjustSpeed sprite, d
			sprite\position#[d] = sprite\position#[d] + sprite\speed#[d]
		Next
	End If
End Function

Function setSpeedStepAll(sprite.typeSprite, n#)
	For d = dMin To dMax
		setSpeedStep sprite, d, n#
	Next
End Function

Function setSpeedStep(sprite.typeSprite, d, n#)
	sprite\speedStep#[d] = n#
End Function

Function spriteAdjustSpeed(sprite.typeSprite, d)
	If sprite\speed#[d] > sprite\speedMax#[d] Then
		sprite\speed#[d] = sprite\speedMax#[d]
	ElseIf sprite\speed#[d] < -sprite\speedMax#[d] Then
		sprite\speed#[d] = -sprite\speedMax#[d]
	End If
End Function

Function spriteBreakSpeed(sprite.typeSprite, d)
	If sprite\speed#[d] > 0 Then
		sprite\speed#[d] = sprite\speed#[d] - sprite\speedBreak#[d]
		If sprite\speed#[d] < 0 Then
			sprite\speed#[d] = 0
		End If
	ElseIf sprite\speed#[d] < 0 Then
		sprite\speed#[d] = sprite\speed#[d] + sprite\speedBreak#[d]
		If sprite\speed#[d] > 0 Then
			sprite\speed#[d] = 0
		End If
	End If
End Function

Function spriteStopSpeed(sprite.typeSprite)
	For d = dMin To dMax
		sprite\speed#[d] = 0
	Next
End Function

Function setSpeedMaxAll(sprite.typeSprite, n#)
	For d = dMin To dMax
		setSpeedMax sprite.typeSprite, d, n#
	Next
End Function

Function setSpeedMax(sprite.typeSprite, d, n#)
	sprite\speedMax#[d] = n#
End Function

Function setSpeedBreakAll(sprite.typeSprite, n#)
	setSpeedBreak sprite.typeSprite, n#, n#, n#
End Function

Function setSpeedBreak(sprite.typeSprite, x#, y#, z#)
	sprite\speedBreak#[dX] = x#
	sprite\speedBreak#[dY] = y#
	sprite\speedBreak#[dZ] = z#
End Function

Function moveSpeed(sprite.typeSprite, d, n#)
	sprite\speed#[d] = sprite\speed#[d] + n#
End Function

Function adaptFall(sprite.typeSprite)
	sprite\speed#[dY] = sprite\speed#[dY] - .4
End Function

Function setPositionXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\position#[dX] = x#
	sprite\position#[dY] = y#
	sprite\position#[dZ] = z#
End Function

Function setPosition(sprite.typeSprite, d, n#)
	sprite\position#[d] = n#
End Function

Function moveX(sprite.typeSprite, n#)
	setX sprite, getX#(sprite) + n#
End Function

Function moveY(sprite.typeSprite, n#)
	setY sprite, getY#(sprite) + n#
End Function

Function moveZ(sprite.typeSprite, n#)
	setZ sprite, getZ#(sprite) + n#
End Function

Function moveXYZ(sprite.typeSprite, x#, y#, z#)
	moveX sprite, x#
	moveY sprite, y#
	moveZ sprite, z#
End Function

Function getX#(sprite.typeSprite)
	Return sprite\position#[dX]
End Function

Function getY#(sprite.typeSprite)
	Return sprite\position#[dY]
End Function

Function getZ#(sprite.typeSprite)
	Return sprite\position#[dZ]
End Function

Function setX(sprite.typeSprite, n#)
	sprite\position#[dX] = n#
End Function

Function setY(sprite.typeSprite, n#)
	sprite\position#[dY] = n#
End Function

Function setZ(sprite.typeSprite, n#)
	sprite\position#[dZ] = n#
End Function

Function setTexture(sprite.typeSprite, fileName$)
	setTextureWithFilter sprite, fileName$, gTextureColor + gTextureMipmapped
End Function

Function setTextureWithFilter(sprite.typeSprite, fileName$, filter)
	sprite\texture = LoadTexture(fileName$, filter)
	adaptTexture sprite
End Function

Function setScaleXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\scale#[dX] = x#
	sprite\scale#[dY] = y#
	sprite\scale#[dZ] = z#
End Function

Function moveScaleXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\scale#[dX] = sprite\scale#[dX] + x#
	sprite\scale#[dY] = sprite\scale#[dY] + y#
	sprite\scale#[dZ] = sprite\scale#[dZ] + z#
End Function

Function moveScaleAll(sprite.typeSprite, n#)
	For d = dMin To dMax
		sprite\scale#[d] = sprite\scale#[d] + n#
	Next
End Function

Function setScaleAll(sprite.typeSprite, n#)
	For d = dMin To dMax
		sprite\scale#[d] = n#
	Next
End Function

Function setScale(sprite.typeSprite, d, n#)
	sprite\scale#[d] = n#
End Function

Function copyScale(reference.typeSprite, sprite.typeSprite)
	For d = dMin To dMax
		sprite\scale#[d] = reference\scale#[d]
	Next
End Function

Function copyColor(reference.typeSprite, sprite.typeSprite)
	For c = colMin To colMax
		sprite\col[c] = reference\col[c]
	Next
End Function

Function copyAngle(reference.typeSprite, sprite.typeSprite)
	For d = dMin To dMax
		sprite\angle#[d] = reference\angle#[d]
	Next
End Function

Function copyPosition(reference.typeSprite, sprite.typeSprite)
	For d = dMin To dMax
		sprite\position#[d] = reference\position#[d]
	Next
End Function

Function setAngleXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\angle#[dX] = x#
	sprite\angle#[dY] = y#
	sprite\angle#[dZ] = z#
End Function

Function moveAngleXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\angle#[dX] = sprite\angle#[dX] + x#
	sprite\angle#[dY] = sprite\angle#[dY] + y#
	sprite\angle#[dZ] = sprite\angle#[dZ] + z#
End Function

Function moveAngleTargetXYZ(sprite.typeSprite, x#, y#, z#)
	sprite\angleTarget#[dX] = sprite\angleTarget#[dX] + x#
	sprite\angleTarget#[dY] = sprite\angleTarget#[dY] + y#
	sprite\angleTarget#[dZ] = sprite\angleTarget#[dZ] + z#
End Function

Function setAngle(sprite.typeSprite, d, n#)
	sprite\angle#[d] = n#
End Function

Function adaptPosition(sprite.typeSprite)
	If sprite\isMesh Then
		PositionMesh sprite\body, sprite\position#[dX], sprite\position#[dY], sprite\position#[dZ]
	Else
		PositionEntity sprite\body, sprite\position#[dX], sprite\position#[dY], sprite\position#[dZ]
	End If
End Function

Function adaptTexture(sprite.typeSprite)
	If sprite\texture <> 0
		EntityTexture sprite\body, sprite\texture
	EndIf
End Function

Function adaptScale(sprite.typeSprite)
	If sprite\class = classParticle Then
		ScaleSprite sprite\body, sprite\scale#[dX], sprite\scale#[dY]
	Else
		If sprite\isMesh Then
			ScaleMesh sprite\body, sprite\scale#[dX], sprite\scale#[dY], sprite\scale#[dZ]
		Else
			ScaleEntity sprite\body, sprite\scale#[dX], sprite\scale#[dY], sprite\scale#[dZ]
		End If
	End If
End Function

Function adaptAngle(sprite.typeSprite)
	angleFound = False
	For d = dMin To dMax
		If sprite\angle#[d] <> nothing And sprite\angle#[d] <> 0 Then
			angleFound = True
		End If
	Next
	If angleFound Then
		If sprite\isMesh Then
			RotateMesh sprite\body, sprite\angle#[dX], sprite\angle#[dY], sprite\angle#[dZ]
		Else
			RotateEntity sprite\body, sprite\angle#[dX], sprite\angle#[dY], sprite\angle#[dZ]
		End If
	End If
End Function

Function clearAngle(sprite.typeSprite)
	For d = dMin To dMax
		sprite\angle#[d] = 0
	Next
End Function

Function clearColor(sprite.typeSprite)
	For c = colMin To colMax
		sprite\col[c] = nothing
	Next
End Function

Function clearColorTarget(sprite.typeSprite)
	For c = colMin To colMax
		sprite\colTarget[c] = nothing
	Next
End Function

Function setSpriteColorAll(sprite.typeSprite, n)
	For c = colMin To colMax
		sprite\col[c] = n
	Next
End Function

Function setSpriteColorRGB(sprite.typeSprite, red, green, blue)
	sprite\col[cRed] = red
	sprite\col[cGreen] = green
	sprite\col[cBlue] = blue
End Function

Function moveSpriteColorRGB(sprite.typeSprite, red, green, blue)
	sprite\col[cRed] = sprite\col[cRed] + red
	sprite\col[cGreen] = sprite\col[cGreen] + green
	sprite\col[cBlue] = sprite\col[cBlue] + blue
End Function

Function setSpriteColorTargetAll(sprite.typeSprite, n)
	For c = colMin To colMax
		sprite\colTarget[c] = n
	Next
End Function

Function setSpriteColorTargetRGB(sprite.typeSprite, red, green, blue)
	sprite\colTarget[cRed] = red
	sprite\colTarget[cGreen] = green
	sprite\colTarget[cBlue] = blue
End Function

Function moveSpriteColorTargetRGB(sprite.typeSprite, red, green, blue)
	sprite\colTarget[cRed] = sprite\colTarget[cRed] + red
	sprite\colTarget[cGreen] = sprite\colTarget[cGreen] + green
	sprite\colTarget[cBlue] = sprite\colTarget[cBlue] + blue
End Function

Function moveSpriteColorAll(sprite.typeSprite, n)
	For c = colMin To colMax
		sprite\col[c] = sprite\col[c] + n
	Next
End Function

Function moveSpriteColorTargetAll(sprite.typeSprite, n)
	For c = colMin To colMax
		sprite\colTarget[c] = sprite\colTarget[c] + n
	Next
End Function

Function setScaleTargetBySprite(sprite.typeSprite, reference.typeSprite)
	For d = dMin To dMax
		sprite\scaleTarget#[d] = reference\scale#[d]
	Next
End Function

Function setPositionTargetBySprite(sprite.typeSprite, n, reference.typeSprite, offX#, offY#, offZ#)
	If n = 0 Then
		sprite\positionTarget0#[dX] = reference\position#[dX] + offX#
		sprite\positionTarget0#[dY] = reference\position#[dY] + offY#
		sprite\positionTarget0#[dZ] = reference\position#[dZ] + offZ#
	ElseIf n = 1 Then
		sprite\positionTarget1#[dX] = reference\position#[dX] + offX#
		sprite\positionTarget1#[dY] = reference\position#[dY] + offY#
		sprite\positionTarget1#[dZ] = reference\position#[dZ] + offZ#
	End If
End Function

Function setScaleTargetAll(sprite.typeSprite, n#)
	For d = dMin To dMax
		sprite\scaleTarget#[d] = n#
	Next
End Function

Function clearScaleTarget(sprite.typeSprite)
	For d = dMin To dMax
		sprite\scaleTarget#[d] = nothing
	Next
End Function

Function clearPositionTarget(sprite.typeSprite)
	sprite\positionTargetFollow = nothing
	For d = dMin To dMax
		sprite\positionTarget0#[d] = nothing
		sprite\positionTarget1#[d] = nothing
	Next
End Function

Function clearAngleTarget(sprite.typeSprite)
	For d = dMin To dMax
		sprite\angleTarget#[d] = nothing
	Next
End Function

Function countClass(ofClass)
	counter = 0
	For sprite.typeSprite = Each typeSprite
		If sprite\class = ofClass Then
			counter = counter + 1
		End If
	Next
	
	Return counter
End Function

Function reachedAngleTarget(sprite.typeSprite)
	didReach = True

	For d = dMin To dMax
		If sprite\angleTarget#[d] <> nothing Then
			If sprite\angle#[d] <> sprite\angleTarget#[d] Then
				didReach = False
				Exit
			End If
		End If
	Next
	
	Return didReach
End Function

Function dropShadow(dropper.typeSprite, toClass)
	sprite.typeSprite = New typeSprite
	initSprite sprite
	sprite\body = CreateSphere()
	sprite\class = classShadow
	EntityType sprite\body, sprite\class
	
	x# = dropper\position#[dX]
	z# = dropper\position#[dZ]
	yMin# = sprite\position#[dY]
	yMax# = yMin# - 9
	
	Collisions classShadow, classTile, gCollisionMethodSphereToPolygon, gCollisionResponseStop
		
	For y# = yMin# To yMax# Step -.7
		PositionEntity sprite\body, x#, y#, z#
		UpdateWorld
		
		If EntityCollided(sprite\body, toClass) Then
			For i = 1 To CountCollisions(sprite\body)
				randomAmount# = .7
				randomAmountHalf# = randomAmount# / 2
				entityInLine = CollisionEntity(sprite\body, i)
				spriteInLine.typeSprite = getSpriteOfBody.typeSprite(entityInLine)
				For n = 1 To 3
					pX# = dropper\position#[dX]
					pY# = spriteInLine\position#[dY] + 1
					pZ# = dropper\position#[dZ]
					pX# = pX# + Rnd(randomAmount#) - randomAmountHalf#
					pZ# = pZ# + Rnd(randomAmount#) - randomAmountHalf#
					createShadowParticle pX#, pY#, pZ#
				Next
			Next
		End If	
	Next
	
	FreeEntity sprite\body
	Delete sprite
End Function

Function followPositionTarget(sprite.typeSprite)
	n = sprite\positionTargetFollow
	If n <> nothing Then

		If n = 0 Then
		
			For d = dMin To dMax
				If sprite\positionTarget0#[d] <> nothing Then
					If sprite\position#[d] < sprite\positionTarget0#[d] Then
						sprite\speed#[d] = sprite\speed#[d] + sprite\speedStep#[d]
						If sprite\position#[d] + sprite\speed#[d] > sprite\positionTarget0#[d] Then
							sprite\position#[d] = sprite\positionTarget0#[d]
							sprite\speed#[d] = 0
						End If
					ElseIf sprite\position#[d] > sprite\positionTarget0#[d] Then
						sprite\speed#[d] = sprite\speed#[d] - sprite\speedStep#[d]
						If sprite\position#[d] + sprite\speed#[d] < sprite\positionTarget0#[d] Then
							sprite\position#[d] = sprite\positionTarget0#[d]
							sprite\speed#[d] = 0
						End If
					End If
				End If
			Next
			
		ElseIf n = 1 Then

			For d = dMin To dMax
				If sprite\positionTarget1#[d] <> nothing Then
					If sprite\position#[d] < sprite\positionTarget1#[d] Then
						sprite\speed#[d] = sprite\speed#[d] + sprite\speedStep#[d]
						If sprite\position#[d] + sprite\speed#[d] > sprite\positionTarget1#[d] Then
							sprite\position#[d] = sprite\positionTarget1#[d]
							sprite\speed#[d] = 0
						End If
					ElseIf sprite\position#[d] > sprite\positionTarget1#[d] Then
						sprite\speed#[d] = sprite\speed#[d] - sprite\speedStep#[d]
						If sprite\position#[d] + sprite\speed#[d] < sprite\positionTarget1#[d] Then
							sprite\position#[d] = sprite\positionTarget1#[d]
							sprite\speed#[d] = 0
						End If
					End If
				End If
			Next

		End If
		
		If reachedPositionTarget(sprite.typeSprite, n) Then
			n = n + 1
			If n > 1 Then
				n = 0
			End If
			sprite\positionTargetFollow = n
		End If
		
	End If
End Function

Function reachedPositionTarget(sprite.typeSprite, n)
	didReach = True

	If n = 0 Then
		For d = dMin To dMax
			If sprite\positionTarget0#[d] <> nothing Then
				If sprite\position#[d] <> sprite\positionTarget0#[d] Then
					didReach = False
					Exit
				End If
			End If
		Next
	ElseIf n = 1 Then
		For d = dMin To dMax
			If sprite\positionTarget1#[d] <> nothing Then
				If sprite\position#[d] <> sprite\positionTarget1#[d] Then
					didReach = False
					Exit
				End If
			End If
		Next
	End If
	
	Return didReach
End Function

Function clearSpeed(sprite.typeSprite)
	For d = dMin To dMax
		sprite\speed#[d] = 0
		sprite\speedMax#[d] = 2
		sprite\speedBreak#[d] = .1
		sprite\speedStep#[d] = .5
	Next
End Function